/* generated by Makefile.PL for perl 5.011002 */
PL_ppaddr[OP_ACCEPT] = pp_slowop_profiler;
PL_ppaddr[OP_BACKTICK] = pp_slowop_profiler;
PL_ppaddr[OP_BIND] = pp_slowop_profiler;
PL_ppaddr[OP_BINMODE] = pp_slowop_profiler;
PL_ppaddr[OP_CHDIR] = pp_slowop_profiler;
PL_ppaddr[OP_CHMOD] = pp_slowop_profiler;
PL_ppaddr[OP_CHOWN] = pp_slowop_profiler;
PL_ppaddr[OP_CHROOT] = pp_slowop_profiler;
PL_ppaddr[OP_CLOSE] = pp_slowop_profiler;
PL_ppaddr[OP_CLOSEDIR] = pp_slowop_profiler;
PL_ppaddr[OP_CONNECT] = pp_slowop_profiler;
PL_ppaddr[OP_CRYPT] = pp_slowop_profiler;
PL_ppaddr[OP_DBMCLOSE] = pp_slowop_profiler;
PL_ppaddr[OP_DBMOPEN] = pp_slowop_profiler;
PL_ppaddr[OP_DUMP] = pp_slowop_profiler;
PL_ppaddr[OP_EGRENT] = pp_slowop_profiler;
PL_ppaddr[OP_EHOSTENT] = pp_slowop_profiler;
PL_ppaddr[OP_ENETENT] = pp_slowop_profiler;
PL_ppaddr[OP_ENTERWRITE] = pp_slowop_profiler;
PL_ppaddr[OP_EOF] = pp_slowop_profiler;
PL_ppaddr[OP_EPROTOENT] = pp_slowop_profiler;
PL_ppaddr[OP_EPWENT] = pp_slowop_profiler;
PL_ppaddr[OP_ESERVENT] = pp_slowop_profiler;
PL_ppaddr[OP_FCNTL] = pp_slowop_profiler;
PL_ppaddr[OP_FLOCK] = pp_slowop_profiler;
PL_ppaddr[OP_FORMLINE] = pp_slowop_profiler;
PL_ppaddr[OP_FTATIME] = pp_slowop_profiler;
PL_ppaddr[OP_FTBINARY] = pp_slowop_profiler;
PL_ppaddr[OP_FTBLK] = pp_slowop_profiler;
PL_ppaddr[OP_FTCHR] = pp_slowop_profiler;
PL_ppaddr[OP_FTCTIME] = pp_slowop_profiler;
PL_ppaddr[OP_FTDIR] = pp_slowop_profiler;
PL_ppaddr[OP_FTEEXEC] = pp_slowop_profiler;
PL_ppaddr[OP_FTEOWNED] = pp_slowop_profiler;
PL_ppaddr[OP_FTEREAD] = pp_slowop_profiler;
PL_ppaddr[OP_FTEWRITE] = pp_slowop_profiler;
PL_ppaddr[OP_FTFILE] = pp_slowop_profiler;
PL_ppaddr[OP_FTIS] = pp_slowop_profiler;
PL_ppaddr[OP_FTLINK] = pp_slowop_profiler;
PL_ppaddr[OP_FTMTIME] = pp_slowop_profiler;
PL_ppaddr[OP_FTPIPE] = pp_slowop_profiler;
PL_ppaddr[OP_FTREXEC] = pp_slowop_profiler;
PL_ppaddr[OP_FTROWNED] = pp_slowop_profiler;
PL_ppaddr[OP_FTRREAD] = pp_slowop_profiler;
PL_ppaddr[OP_FTRWRITE] = pp_slowop_profiler;
PL_ppaddr[OP_FTSGID] = pp_slowop_profiler;
PL_ppaddr[OP_FTSIZE] = pp_slowop_profiler;
PL_ppaddr[OP_FTSOCK] = pp_slowop_profiler;
PL_ppaddr[OP_FTSUID] = pp_slowop_profiler;
PL_ppaddr[OP_FTSVTX] = pp_slowop_profiler;
PL_ppaddr[OP_FTTEXT] = pp_slowop_profiler;
PL_ppaddr[OP_FTTTY] = pp_slowop_profiler;
PL_ppaddr[OP_FTZERO] = pp_slowop_profiler;
PL_ppaddr[OP_GETC] = pp_slowop_profiler;
PL_ppaddr[OP_GETLOGIN] = pp_slowop_profiler;
PL_ppaddr[OP_GETPEERNAME] = pp_slowop_profiler;
PL_ppaddr[OP_GETSOCKNAME] = pp_slowop_profiler;
PL_ppaddr[OP_GGRENT] = pp_slowop_profiler;
PL_ppaddr[OP_GGRGID] = pp_slowop_profiler;
PL_ppaddr[OP_GGRNAM] = pp_slowop_profiler;
PL_ppaddr[OP_GHBYADDR] = pp_slowop_profiler;
PL_ppaddr[OP_GHBYNAME] = pp_slowop_profiler;
PL_ppaddr[OP_GHOSTENT] = pp_slowop_profiler;
PL_ppaddr[OP_GLOB] = pp_slowop_profiler;
PL_ppaddr[OP_GNBYADDR] = pp_slowop_profiler;
PL_ppaddr[OP_GNBYNAME] = pp_slowop_profiler;
PL_ppaddr[OP_GNETENT] = pp_slowop_profiler;
PL_ppaddr[OP_GPBYNAME] = pp_slowop_profiler;
PL_ppaddr[OP_GPBYNUMBER] = pp_slowop_profiler;
PL_ppaddr[OP_GPROTOENT] = pp_slowop_profiler;
PL_ppaddr[OP_GPWENT] = pp_slowop_profiler;
PL_ppaddr[OP_GPWNAM] = pp_slowop_profiler;
PL_ppaddr[OP_GPWUID] = pp_slowop_profiler;
PL_ppaddr[OP_GSBYNAME] = pp_slowop_profiler;
PL_ppaddr[OP_GSBYPORT] = pp_slowop_profiler;
PL_ppaddr[OP_GSERVENT] = pp_slowop_profiler;
PL_ppaddr[OP_GSOCKOPT] = pp_slowop_profiler;
PL_ppaddr[OP_IOCTL] = pp_slowop_profiler;
PL_ppaddr[OP_LEAVEWRITE] = pp_slowop_profiler;
PL_ppaddr[OP_LINK] = pp_slowop_profiler;
PL_ppaddr[OP_LISTEN] = pp_slowop_profiler;
PL_ppaddr[OP_LOCK] = pp_slowop_profiler;
PL_ppaddr[OP_LSTAT] = pp_slowop_profiler;
PL_ppaddr[OP_MATCH] = pp_slowop_profiler;
PL_ppaddr[OP_MKDIR] = pp_slowop_profiler;
PL_ppaddr[OP_MSGCTL] = pp_slowop_profiler;
PL_ppaddr[OP_MSGGET] = pp_slowop_profiler;
PL_ppaddr[OP_MSGRCV] = pp_slowop_profiler;
PL_ppaddr[OP_MSGSND] = pp_slowop_profiler;
PL_ppaddr[OP_OPEN] = pp_slowop_profiler;
PL_ppaddr[OP_OPEN_DIR] = pp_slowop_profiler;
PL_ppaddr[OP_PACK] = pp_slowop_profiler;
PL_ppaddr[OP_PRINT] = pp_slowop_profiler;
PL_ppaddr[OP_PRTF] = pp_slowop_profiler;
PL_ppaddr[OP_QR] = pp_slowop_profiler;
PL_ppaddr[OP_RCATLINE] = pp_slowop_profiler;
PL_ppaddr[OP_READ] = pp_slowop_profiler;
PL_ppaddr[OP_READDIR] = pp_slowop_profiler;
PL_ppaddr[OP_READLINE] = pp_slowop_profiler;
PL_ppaddr[OP_READLINK] = pp_slowop_profiler;
PL_ppaddr[OP_RECV] = pp_slowop_profiler;
PL_ppaddr[OP_REGCOMP] = pp_slowop_profiler;
PL_ppaddr[OP_RENAME] = pp_slowop_profiler;
PL_ppaddr[OP_REWINDDIR] = pp_slowop_profiler;
PL_ppaddr[OP_RMDIR] = pp_slowop_profiler;
#if (PERL_VERSION >= 10)
PL_ppaddr[OP_SAY] = pp_slowop_profiler;
#endif
PL_ppaddr[OP_SEEK] = pp_slowop_profiler;
PL_ppaddr[OP_SEEKDIR] = pp_slowop_profiler;
PL_ppaddr[OP_SELECT] = pp_slowop_profiler;
PL_ppaddr[OP_SEMCTL] = pp_slowop_profiler;
PL_ppaddr[OP_SEMGET] = pp_slowop_profiler;
PL_ppaddr[OP_SEMOP] = pp_slowop_profiler;
PL_ppaddr[OP_SEND] = pp_slowop_profiler;
PL_ppaddr[OP_SGRENT] = pp_slowop_profiler;
PL_ppaddr[OP_SHMCTL] = pp_slowop_profiler;
PL_ppaddr[OP_SHMGET] = pp_slowop_profiler;
PL_ppaddr[OP_SHMREAD] = pp_slowop_profiler;
PL_ppaddr[OP_SHMWRITE] = pp_slowop_profiler;
PL_ppaddr[OP_SHOSTENT] = pp_slowop_profiler;
PL_ppaddr[OP_SHUTDOWN] = pp_slowop_profiler;
PL_ppaddr[OP_SLEEP] = pp_slowop_profiler;
PL_ppaddr[OP_SNETENT] = pp_slowop_profiler;
PL_ppaddr[OP_SOCKET] = pp_slowop_profiler;
PL_ppaddr[OP_SORT] = pp_slowop_profiler;
PL_ppaddr[OP_SPROTOENT] = pp_slowop_profiler;
PL_ppaddr[OP_SPWENT] = pp_slowop_profiler;
PL_ppaddr[OP_SSELECT] = pp_slowop_profiler;
PL_ppaddr[OP_SSERVENT] = pp_slowop_profiler;
PL_ppaddr[OP_SSOCKOPT] = pp_slowop_profiler;
PL_ppaddr[OP_STAT] = pp_slowop_profiler;
PL_ppaddr[OP_SUBST] = pp_slowop_profiler;
PL_ppaddr[OP_SUBSTCONT] = pp_slowop_profiler;
PL_ppaddr[OP_SYMLINK] = pp_slowop_profiler;
PL_ppaddr[OP_SYSCALL] = pp_slowop_profiler;
PL_ppaddr[OP_SYSOPEN] = pp_slowop_profiler;
PL_ppaddr[OP_SYSREAD] = pp_slowop_profiler;
PL_ppaddr[OP_SYSSEEK] = pp_slowop_profiler;
PL_ppaddr[OP_SYSTEM] = pp_slowop_profiler;
PL_ppaddr[OP_SYSWRITE] = pp_slowop_profiler;
PL_ppaddr[OP_TELL] = pp_slowop_profiler;
PL_ppaddr[OP_TELLDIR] = pp_slowop_profiler;
PL_ppaddr[OP_TRUNCATE] = pp_slowop_profiler;
PL_ppaddr[OP_UMASK] = pp_slowop_profiler;
PL_ppaddr[OP_UNLINK] = pp_slowop_profiler;
PL_ppaddr[OP_UNPACK] = pp_slowop_profiler;
PL_ppaddr[OP_UTIME] = pp_slowop_profiler;
PL_ppaddr[OP_WAIT] = pp_slowop_profiler;
PL_ppaddr[OP_WAITPID] = pp_slowop_profiler;
